<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\EventListener;

use Dedi\SyliusMarketingPlugin\Factory\Events\CartEventFactoryInterface;
use Dedi\SyliusMarketingPlugin\Storage\EventsBag;
use Doctrine\Bundle\DoctrineBundle\EventSubscriber\EventSubscriberInterface;
use Doctrine\ORM\Events;
use Doctrine\Persistence\Event\LifecycleEventArgs;
use Sylius\Component\Order\Model\OrderInterface;

class CartDoctrineEventListener implements EventSubscriberInterface
{
    private EventsBag $eventsStorage;
    private CartEventFactoryInterface $cartEventFactory;

    public function __construct(EventsBag $eventsStorage, CartEventFactoryInterface $cartEventFactory)
    {
        $this->eventsStorage = $eventsStorage;
        $this->cartEventFactory = $cartEventFactory;
    }
    public function getSubscribedEvents(): array
    {
        return [
            Events::preRemove,
        ];
    }

    public function preRemove(LifecycleEventArgs $event): void
    {
        $object = $event->getObject();

        if (!$object instanceof OrderInterface || OrderInterface::STATE_CART !== $object->getState()) {
            return;
        }

        $this->eventsStorage->setEvent(
            $this->cartEventFactory->clearCart($object),
        );
    }
}
